@@ -24,9 +24,11 @@
 #include "libMRML/include/my_throw.h"
 #include "libMRML/include/my_assert.h"
 #include "libMRML/include/my_diagnose.h"
-
-#include <stdio.h>
-#include "expat/xmlparse/xmlparse.h"
+#include "libMRML/include/GIFTExceptions.h"
+#include <cstdio>
+#include <iterator>
+//#include "expat/xmlparse/xmlparse.h"
+#include <expat.h>
 #include "libGIFTQuInvertedFile/include/CQInvertedFile.h"
 #include "libMRML/include/CSessionManager.h"
 #include "libMRML/include/CAttributeList.h"
@@ -41,6 +43,17 @@
 #include "libMRML/include/directory.h" // directorys as determined by ./configure
 #include "libMRML/include/createErrorMessage.h"
 
+#include "libMRML/include/gift-config.h"
+
+#ifdef HAVE_LIBUUID 
+extern "C"{
+  typedef unsigned char* uuid_t ;
+  void uuid_generate(uuid_t); 
+  void uuid_unparse(uuid_t,char*);
+}
+#endif
+
+
 //the "ID administration subsystem"
 TID gID;
 
@@ -177,16 +190,26 @@ void endConfigElement(void *userData, const char *inElementName)
 #endif
 }
 
-#include <time.h>
+#include <ctime>
 #include "libMRML/include/CSessionManager.h"
 #include "libMRML/include/TID.h"
 #include <iostream>
 extern TID gID;
 
 string getNewID(){
+#ifdef HAVE_LIBUUID 
+  char lUUIDString[200];
+  unsigned char lUUIDBinary[16];
+  uuid_t lUUID(lUUIDBinary);
+  uuid_generate(lUUID); 
+  uuid_unparse(lUUID,
+	       lUUIDString);
+  return lUUIDString;
+#else
     char lID[20];
     sprintf(lID,"%ld",++gID);
     return lID;
+#endif
 }
 
 /** */
@@ -257,13 +280,41 @@ string CSession::toXML(bool isPrivate)const{
   return lHelper.toString();
 }
 
+void CSession::clearLanguages(){
+  mLanguages.clear();
+}
+void CSession::addLanguage(string inLanguageCode){
+  mLanguages.push_back(inLanguageCode);
+}
+/** commit the list of languages. That means, here the
+    actual language that will be used throughout the 
+    translation is determined */
+void CSession::commitLanguages(const CI18nTranslator& inTranslator){  
+  for(CLanguageList::const_iterator i=mLanguages.begin();
+      i!=mLanguages.end();
+      i++){
+    if(inTranslator.hasLanguage(*i)){
+      mPreferredLanguage=*i;
+      break;
+    }
+  }
+};
+
+list<string> CSession::getLanguages()const{
+  return mLanguages;
+}
+string CSession::getPreferredLanguage()const{
+  return mPreferredLanguage;
+}
+
 //
 CSession::CSession():
   mActiveAlgorithm(0),
   mSessionName(""),
   mID(""),
   mIsOpen(0),
-  mUser("")
+  mUser(""),
+  mPreferredLanguage("de")
 {
   mMutexSession.lock();
   // nothing: use it with read
@@ -276,7 +327,8 @@ CSession::CSession():
 CSession::CSession(string inUser,
 		   string inID,
 		   string inSessionName,
-		   CAlgorithm* inActiveAlgorithm)
+		   CAlgorithm* inActiveAlgorithm):
+  mPreferredLanguage("de")
 {
   // mutex not needed. if two call this 
   // at the same time, there is some
@@ -413,15 +465,18 @@ CXMLElement* CSession::query(CSessionManager&     inCaller,
     }
   }
   catch(GIFTException* inException){
+    cerr << "We have caught a GIFTException" << endl;
+    cout << "We have caught a GIFTException" << endl;
     return createErrorMessage(inException->getMessage());
   }
   catch(GIFTException& inException){
+    cerr << "We have caught a GIFTException" << endl;
+    cout << "We have caught a GIFTException" << endl;
     return createErrorMessage(inException.getMessage());
   }
   catch(...){
-    cout << "something caught" << endl;
-    cerr << "something caught" << endl;
-    
+    cout << "We have caught a non-GIFTException" << endl;
+    cerr << "We have caught a non-GIFTException" << endl;
     exit(10);
   }
 };
@@ -451,11 +506,13 @@ ostream& operator<<(ostream& outStream,
 
 ///
 CSessionManager::CSessionManager(string inSessionFileName,
-				 string inConfigFileName):
+				 string inConfigFileName,
+				 string inI18nFileName):
   mPropertySheetList(0),
   mPropertySheetSubtree(0),
   mAccessorAdminCollection(inConfigFileName),
   CAlgorithmCollection(inConfigFileName),
+  mI18nTranslator(inI18nFileName),
   mBaseTypeFactory(new CDynamicQueryFactory(__LIBDIR__)){
 
   ifstream lSessions(inSessionFileName.c_str());
@@ -495,7 +552,7 @@ CSessionManager::CSessionManager(string inSessionFileName,
 	   << " at line "
 	   << XML_GetCurrentLineNumber(lParser)
 	   << endl;
-      exit(1);
+      exit(1);// this happens only when parsing the config file
     }
   } while (!lDone);
 
@@ -970,22 +1027,116 @@ bool CSessionManager::setAlgorithm(const string& inSessionID,
        << flush
        << endl;
 
-  CSession* lSession=mIDToSession[inSessionID];
-
+  CIDToSession::const_iterator lFound(mIDToSession.find(inSessionID));
   
   //  assert(lSession);
-  if(!lSession){
-    //create a session with the required sessionID
-    this->newSession(inSessionID,
-		     "I-dunno",
-		     "I-know-even-less");
-		     
-  }
+  if(lFound==mIDToSession.end()){
 
-  bool lResult=lSession->setActiveAlgorithm(mAccessorAdminCollection,
- 					    *this,
- 					    inAlgorithm,
- 					    *mBaseTypeFactory);
+    my_throw(VENotFound("Could not find Session "));
+
+    //
+    // the following code has been deleted, as it would
+    // generate a security problem: you could flood the GIFT
+    // by querying with a host of unknown session IDs
+    //
+    //     //create a session with the required sessionID
+    //     lSession=mIDToSession[this->newSession(inSessionID,
+    // 					   "I-dunno",
+    // 					   "I-know-even-less")];
+    //     // if this fails this is a real programming mistake
+    //     // it should never happen
+    //     // please don't delete
+    //     assert(lSession);
+  }
+  // if this fails this is a real bug
+  // it should never happen
+  // please don't delete
+  assert(lFound->second);
+    
+  bool lResult=lFound->second->setActiveAlgorithm(mAccessorAdminCollection,
+						  *this,
+						  inAlgorithm,
+						  *mBaseTypeFactory);
+  mMutexSessionManager.unlock();
+  return lResult;
+};
+bool CSessionManager::clearSessionLanguages(const string& inSessionID){
+  mMutexSessionManager.lock();
+  //this should also kick of the 
+  //creation of a suitable query tree
+
+  CIDToSession::const_iterator lFound(mIDToSession.find(inSessionID));
+  
+  //  assert(lSession);
+  if(lFound==mIDToSession.end()){
+    my_throw(VENotFound("Could not find Session"));
+  }
+  // if this fails this is a real bug
+  // it should never happen
+  // please don't delete
+  assert(lFound->second);
+    
+  lFound->second->clearLanguages();
+  mMutexSessionManager.unlock();
+  return true;
+};
+bool CSessionManager::addSessionLanguage(const string& inSessionID,
+					 const string& inLanguageCode){
+  mMutexSessionManager.lock();
+  //this should also kick of the 
+  //creation of a suitable query tree
+
+  CIDToSession::const_iterator lFound(mIDToSession.find(inSessionID));
+  
+  //  assert(lSession);
+  if(lFound==mIDToSession.end()){
+    my_throw(VENotFound("Could not find Session"));
+  }
+  // if this fails this is a real bug
+  // it should never happen
+  // please don't delete
+  assert(lFound->second);
+    
+  lFound->second->addLanguage(inLanguageCode);
+  mMutexSessionManager.unlock();
+  return true;
+};
+bool CSessionManager::commitSessionLanguages(const string& inSessionID){
+  mMutexSessionManager.lock();
+  //this should also kick of the 
+  //creation of a suitable query tree
+
+  CIDToSession::const_iterator lFound(mIDToSession.find(inSessionID));
+  
+  //  assert(lSession);
+  if(lFound==mIDToSession.end()){
+    my_throw(VENotFound("Could not find Session"));
+  }
+  // if this fails this is a real bug
+  // it should never happen
+  // please don't delete
+  assert(lFound->second);
+    
+  lFound->second->commitLanguages(mI18nTranslator);
+  mMutexSessionManager.unlock();
+  return true;
+};
+list<string> CSessionManager::getSessionLanguages(const string& inSessionID)const{
+  mMutexSessionManager.lock();
+  //this should also kick of the 
+  //creation of a suitable query tree
+  CIDToSession::const_iterator lFound(mIDToSession.find(inSessionID));
+  
+  //  assert(lSession);
+  if(lFound==mIDToSession.end()){
+    my_throw(VENotFound("Could not find Session"));
+  }
+  // if this fails this is a real bug
+  // it should never happen
+  // please don't delete
+  assert(lFound->second);
+    
+  list<string> lResult=lFound->second->getLanguages();
   mMutexSessionManager.unlock();
   return lResult;
 };
@@ -995,35 +1146,44 @@ CXMLElement* CSessionManager::query(const string& inSessionID,
   // this is function not completely locked, because the functions called do not neccessarily 
   // need a lock. LOCKING THIS FUNCTION COMPLETELY WOULD MEAN LOSING CONCURRENCY.
   //mMutexSessionManager.lock();//debugging
-  CIDToSession::const_iterator lFound=mIDToSession.find(inSessionID);
+
+  CIDToSession::const_iterator lFound(mIDToSession.find(inSessionID));
   //mMutexSessionManager.unlock();
   //
-  if(lFound==mIDToSession.end()){
-#ifdef GIFT_THROW_UNKNOWN_SESSION
-    cout << "throwing: "
- 	 <<(VEUnknownSession(inSessionID.c_str()))
- 	 << endl
- 	 << flush;
-#else
-    //     my_throw(VEUnknownSession(inSessionID.c_str()));
-    //     cerr << "this line should not be reached" 
-    // 	 << endl;
-    //     assert(0);
-    CXMLElement* lError(new CXMLElement(mrml_const::error,0));
-    lError->addAttribute(mrml_const::message,"Could not process query: unknown session (ID:"+inSessionID+").");
-    return lError;
-#endif
-  }
   try{
-    CXMLElement* lReturnValue(lFound->second->query(*this,
-						    inRelevanceLevelList));
-    //mMutexSessionManager.unlock();//debugging
-    return lReturnValue;
+
+    if(lFound==mIDToSession.end()){
+#ifdef GIFT_THROW_UNKNOWN_SESSION
+//       cout << "throwing: "
+// 	   <<(VEUnknownSession(inSessionID.c_str()))
+// 	   << endl
+// 	   << flush;
+      my_throw(VEUnknownSession((string(Could not process query: unknown session)
+				 +inSessionID).c_str()));
+#else
+      //cout << "NOT throwing: " << endl;
+      //     my_throw(VEUnknownSession(inSessionID.c_str()));
+      //     cerr << "this line should not be reached" 
+      // 	 << endl;
+      //     assert(0);
+      CXMLElement* lError(new CXMLElement(mrml_const::error,0));
+      lError->addAttribute(mrml_const::message,"Could not process query: unknown session (ID:"+inSessionID+").");
+      return lError;
+#endif
+    }else{
+      //cout << "querying: " << endl;
+      CXMLElement* lReturnValue(lFound->second->query(*this,
+						      inRelevanceLevelList));
+      //mMutexSessionManager.unlock();//debugging
+      return lReturnValue;
+    }
   }  
   catch(GIFTException& inCaught){
+    //cout << "CAUGHT3" << endl;
     return createErrorMessage(inCaught); 
   }
   catch(GIFTException* inCaught){
+    //cout << "CAUGHT4" << endl;
     return createErrorMessage(inCaught); 
   }
 };
@@ -1114,9 +1274,32 @@ CXMLElement* CSessionManager::getCollections()const{
   mMutexSessionManager.unlock();
   return lReturnValue;
 };
+/** */
 CXMLElement* CSessionManager::getAlgorithms()const{
   mMutexSessionManager.lock();
   CXMLElement* lReturnValue(((CAlgorithmCollection*)this)->toXMLElement());
   mMutexSessionManager.unlock();
   return lReturnValue;
 };
+/** i18n: get the list of preferred languages of this session */
+void CSessionManager::translate(string inSessionID,
+				CXMLElement& inoutToBeTranslated)const{
+  mMutexSessionManager.lock();
+  //this should also kick of the 
+  //creation of a suitable query tree
+  CIDToSession::const_iterator lFound(mIDToSession.find(inSessionID));
+  
+  //  assert(lSession);
+  if(lFound==mIDToSession.end()){
+    my_throw(VENotFound("Could not find Session"));
+  }
+  // if this fails this is a real bug
+  // it should never happen
+  // please don't delete
+  assert(lFound->second);
+  
+  mI18nTranslator.translateXMLTree(lFound->second->getPreferredLanguage(),
+				   inoutToBeTranslated);
+
+  mMutexSessionManager.unlock();
+};
